//------------------------------------------------------------------------------
// JsonFse - Copyright (c) 2015 by Friedemann Seebass, Germany.
//------------------------------------------------------------------------------
//
// Diese Bibliothek ist freie Software. Sie koennen sie unter den Bedingungen
// der GNU Lesser General Public License Version 2.1, wie von der Free Software
// Foundation veroeffentlicht, weitergeben und/oder modifizieren. 
//
// Die Veroeffentlichung dieser Bibliothek erfolgt in der Hoffnung, dass sie
// Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, insbesondere ohne
// die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT FUER EINEN
// BESTIMMTEN ZWECK. Details finden Sie in der GNU Lesser General Public
// License. 
//
// Sie sollten eine Kopie der GNU Lesser General Public License zusammen mit 
// dieser Bibliothek erhalten haben. Falls nicht, schreiben Sie an die Free 
// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
// USA.
//
//------------------------------------------------------------------------------
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License version
// 2.1 as published by the Free Software Foundation.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//------------------------------------------------------------------------------
/**
 Simple JSON Parser.
 @author Friedemann Seebass
 */// fse, 11.09.2015

package JsonFse;

//------------------------------------------------------------------------------
/**
Simple JSON parser. It parses strings according to the grammer given in the
package documentation. It builds a recurssive tree reflecting the structure
of the JSON object.
@author Friedemann Seebass
 */// fse, 11.09.2015

public class JsonParser
{
	private String mJsonStr = null;  /** zu parsender String                    */
	private int    mPos     = 0;     /** aktuelle Position der Scan-Marke       */
	private int    mSymPos  = 0;     /** Position des zuletzt gelesenen Symbols */

	//---------------------------------------------------------------------------
	/**
	 Creates from the JSON object, given as a String in parameter str, a
	 recursive tree representing the JSON object.
	 @param str the JSON object to parse, given as Java String
	 @return the root node of the created tree
	 @throws ParseErrorEx Is thrown if the parser detects an error in the string.
	 */// fse, 11.09.2015
		
	public NodeObj createTree(String str) throws ParseErrorEx
	{
		mJsonStr = str;
		mPos     = 0;
		NodeObj nodObj = parseJsonObj();
		if (mPos < str.length())
			throw new ParseErrorEx(mJsonStr, mPos, "Unparsed character(s) detected!");
		return nodObj;
	}
	
	//------------------------------------- -------------------------------------
	/**
	Parsed das in mJsonString vorliegende JSON-Objekt und erzeugt daraus den Baum
	der Unterobjekte:
	object = '{' , [ members ] , '}' ;
	@return Wurzelknoten des JSON-Baums
	@throws ParseErrorEx 
	*/// fse, 11.09.2015
	
	private NodeObj parseJsonObj() throws ParseErrorEx
	{
		NodeObj root = new NodeObj();
		String  sym  = getNextSymbol();
		
		if (!sym.equals("{"))
			throw new ParseErrorEx(mJsonStr, mSymPos+1, "'{' expected!");
		sym = getNextSymbol();
		if (sym.equals("}"))
			return root;
		else
		{
			mPos = mSymPos;
			parseMembers(root);
		}
		sym = getNextSymbol();
		if (!sym.equals("}"))
			throw new ParseErrorEx(mJsonStr, mSymPos+1, "'}' expected!");
		
		return root;
	}

	//---------------------------------------------------------------------------
	/**
	Parsed die Members des JSON-Objekts in mJsonStr:
	members = pair , { ',' , pair } ;
	@return Members-Knoten
	@throws ParseErrorEx 
	*/// fse, 12.09.2015
	
	private void parseMembers(NodeObj no) throws ParseErrorEx
	{
//		NodeMembers nm  = new NodeMembers();
		NodeMembers nm  = no.mNodeMembers;
		NodePair    np  = parsePair();
		String      sym;

		nm.mPairList.add(np);
		sym = getNextSymbol();
		while (sym.equals(","))
		{
			np  = parsePair();
			nm.mPairList.add(np);
			sym = getNextSymbol();
		} 
		mPos = mSymPos;
	}
	
	//---------------------------------------------------------------------------
	/**
	Parsed ein Pair eines JSON-Objekts:
	pair = string , ':' , value ;
	@return Knoten eines Pairs
	@throws ParseErrorEx 
	*/// fse, 12.09.2015
	
	private NodePair parsePair() throws ParseErrorEx
	{
		NodePair   np  = new NodePair();
		NodeString ns  = parseString();
		String     sym = getNextSymbol();
		NodeValue  nv;
		
		if (!sym.equals(":"))
			throw new ParseErrorEx(mJsonStr, mSymPos+1, "':' expected!");
		nv = parseValue();
		np.mString = ns;
		np.mValue  = nv;
		return np;
	}
	
	//---------------------------------------------------------------------------
	/**
	Parsed einen JSON-String, also eine durch Anfuehrungszeichen eingeschlossene
	Zeichenkette:
	string = '"' , { char } , '"' ;
	@return Knoten eines Strings
	@throws ParseErrorEx 
	*/// fse, 12.09.2015
	
	private NodeString parseString() throws ParseErrorEx
	{
		String     sym = getNextSymbol();
		NodeString ns  = new NodeString(sym);

		return ns;
	}
	
	//---------------------------------------------------------------------------
	/**
	Parsed einen JSON-Value:
	value = string | number | object | array | 'true' | 'false' | 'null' ;
	@return Knoten eines Values
	@throws ParseErrorEx 
	*/// fse, 13.09.2015
	
	private NodeValue parseValue() throws ParseErrorEx
	{
		NodeValue nv; //  = new NodeValue();
		String    sym = getNextSymbol();
		
		if (sym.charAt(0) == '"')
			nv = new NodeString(sym);
		else if (sym.charAt(0) == '{')
		{
			mPos = mSymPos;
			nv = parseJsonObj();
		}
		else if (sym.charAt(0) == '[')
		{
			mPos = mSymPos;
			nv = parseArray();
		}
		else if (Character.isDigit(sym.charAt(0)) || sym.charAt(0) == '-')
			nv = new NodeNumber(sym);
		else if (sym.equals("true"))
			nv = new NodeTrue();
		else if (sym.equals("false"))
			nv = new NodeFalse();
		else if (sym.equals("null"))
			nv = new NodeNull();
		else
			throw new ParseErrorEx(mJsonStr, mSymPos+1, "Unexpected character!");
		return nv;
	}
	
	//---------------------------------------------------------------------------
	/**
	Parsed ein JSON-Array:
	array = '[' , [ elements ] , ']' ;
	@return Knoten des Arrays
	@throws ParseErrorEx 
	*/// fse, 13.09.2015
	
	private NodeArray parseArray() throws ParseErrorEx
	{
		NodeArray na = new NodeArray();
		mPos = mSymPos;
		String sym = getNextSymbol();
		
		if (!sym.equals("["))
			throw new ParseErrorEx(mJsonStr, mSymPos+1, "'[' expected!");
		sym = getNextSymbol();
		if (sym.equals("]"))
			return na;
		
		mPos = mSymPos;
		parseElements(na.mNodeElements);
		sym = getNextSymbol();
		if (!sym.equals("]"))
			throw new ParseErrorEx(mJsonStr, mSymPos+1, "']' or ',' expected!");
		return na;
	}
	
	//---------------------------------------------------------------------------
	/**
	Parsed die Elemente eines Arrays:
	elements = value , { ',' , value } ;
	@param ne NodeElements-Objekt des NodeArrays.
	@return Knoten der Elemente.
	@throws ParseErrorEx 
	*/// fse, 13.09.2015
	
	private NodeElements parseElements(NodeElements ne) throws ParseErrorEx
	{
		NodeValue    nv  = parseValue();
		String       sym;

		ne.mValueList.add(nv);
		sym = getNextSymbol();
		while (sym.equals(","))
		{
			nv  = parseValue();
			ne.mValueList.add(nv);
			sym = getNextSymbol();
		}
		mPos = mSymPos;
		return ne;
	}
	
	//---------------------------------------------------------------------------
	// Methoden des Scanners:
	//---------------------------------------------------------------------------
	/**
	Sucht ab Position mPos im String mJsonString nach dem naechsten Symbol.
	Leerzeichen werden ueberlesen, am ersten Nicht-Leerzeichen entscheidet sich,
	was fuer ein Symbol gelesen wird: String, Number, "true" oder "false"; alles
	andere sind Einzelzeichen.
	@return das naechste Symbol
	@throws ParseErrorEx 
	*/// fse, 18.09.2015

	private String getNextSymbol() throws ParseErrorEx
	{
		String sym = "";
		String c   = getCurrCharAndIncr(false);

		// Leerzeichen ueberlesen:
		while (c.equals(" ") || c.equals("\t") || c.equals("\n") || c.equals("\r"))
			c = getCurrCharAndIncr(false);
		mSymPos = mPos - 1; // Startposition des nun folgenden Symbols

		// Hauptkategorien finden:
		if (c.equals("\""))
			sym = getString(c);
		else if (Character.isDigit(c.charAt(0)) || c.equals("-"))
			sym = getNumber(c);
		else if (c.equals("t") || c.equals("f") || c.equals("n"))
		{ // true, false oder null:
			sym += c;
			while (true)
			{
				c = getCurrCharAndIncr(false);
				String cs = "" + c;
				if (!"truefalsn".contains(cs))
				{
					mPos--;
					break;
				}
				sym += c;
			}
		}
		else // Einzelzeichen "{", "]" usw.
			sym += c;
//		if (sym.length() <= 1)
//			mPos--;
		return sym;
	}

	//------------------------------------- -------------------------------------
	/**
	Liefert das Zeichen zurueck, auf das mPos in mJasonStr gerade zeigt, und
	inkrementiert mPos. Falls es sich um eine Control-Code-Sequenz handelt,
	werden diese beiden Zeichen zurueckgeliefert.
	Die Control-Code-Zeichen sind der Grund, dass wir einen String zurueckliefern
	muessen und nicht mit einem char auskommen.
	@isToPutBack wenn true, wird mPos zuvor dekrementiert
	@return das einzelne Zeichen mJasonStr.charAt(mPos) als String oder die
	        beiden Zeichen einer Control-Code-Sequenz.
	@throws ParseErrorEx 
	*/// fse, 20.09.2015
	
	private String getCurrCharAndIncr(boolean isToPutBack) throws ParseErrorEx
	{
		char   c;
		String cc;
		
		if (isToPutBack)
			mPos--;
		if (mPos < 0 || mPos >= mJsonStr.length())
			throw new ParseErrorEx(mJsonStr, mPos, "Unexpected access on JSON string at position '" + mPos + "'!");
		c = mJsonStr.charAt(mPos++);
		cc = String.valueOf(c);
		// auf Control-Character pruefen:
		if (c == '\\')
		{
			c = mJsonStr.charAt(mPos++);
			if (c == '\"' || c == '\\' || c == '/' || c == 'b'
			 || c == 'f'  || c == 'n'  || c == 'r' || c == 't' || c == 'u')
				cc += c;
			else
				throw new ParseErrorEx(mJsonStr, mPos, "Unknown control character detected!");
		}
		return cc;
	}
	
	//------------------------------------- -------------------------------------
	/**
	Parsed einen String:
	string = '"' , { char } , '"' ;
	char   = ? any Unicode character ? - ctrlChar - '\') | ctrlChar | hexNumber 
	@return der String incl. An- und Abfuehrungszeichen
	@throws ParseErrorEx 
	*/// fse, 18.09.2015

	private String getString(String c) throws ParseErrorEx
	{
		String sym = "";
		String hn  = "";

		if (!c.equals("\""))
			throw new ParseErrorEx(mJsonStr, mSymPos+1, "'\"' expected!");
		do
		{
			sym += c;
			c = getCurrCharAndIncr(false);
			if (c.equals("\\u"))
			{
				hn = getHexNumber();
				sym += c + hn;
				c = getCurrCharAndIncr(false);
			}
		} while (!c.equals("\""));
		sym += c;
		return sym;
	}
	
	//------------------------ ------------------------- ------------------------
	/**
	Liest einen Control-Character ein.
	ctrlChar  = '\"' | '\\' | '\/' | '\b' | '\f' | '\n' | '\r' | '\t' ;
	@param c der aktuell in Arbeit befindliche char
	@return Der Control-Character.
	@throws ParseErrorEx 
	*/// fse, 25.10.2015
	
//	private static String getCtrlChar(char c) throws ParseErrorEx
//	{
//		String sym = "";
//
//		if (c != '\\')
//			throw new ParseErrorEx(mSymPos+1, "'\\' expected!");
//		sym += c;
//		c = getCurrCharAndIncr(false);
//		if (c != '\"' && c != '\\' && c != '/' && c != 'b' && c != 'f'
//		 && c != 'n'  && c != 'n'  && c != 'r' && c != 't' && c != 'u')
//			throw new ParseErrorEx(mPos, "Unknown ctrl character '" + c + "' detected!");
//		sym += c;
//		return sym;
//	}

	//------------------------ ------------------------- ------------------------
	/**
	Liest vier Hex-Ziffern ein.
	@return Die vier Hex-Ziffern.
	@throws ParseErrorEx 
	*/// fse, 25.10.2015

	private String getHexNumber() throws ParseErrorEx
	{
		String hn = "";
		String allowedChars = "0123456789abcdefABCDEF";
		String c;
		
		for (int i = 0; i < 4; i++)
		{
			c = getCurrCharAndIncr(false);
			if (allowedChars.indexOf(c.charAt(0)) >= 0)
				hn += c;
			else
				throw new ParseErrorEx(mJsonStr, mPos, "'Hex digit expected!");
		}
		return hn;
	}
	
	//------------------------------------- -------------------------------------
	/**
	Parsed eine Number:
	number  = int | int frac | int exp | int , frac , exp ;
	int     = [ '-' ] , digit | digit19 , digits ;
	frac    = '.' , digits ;
	exp     = eE , digits
	eE      = 'e' | 'E' [ '+' | '-' ] ;
	digit09 = digit0 | digit19 ;
	digit0  = '0' ;
	digit19 = '1' | '2' | '3' | '4' | '5' | '6' | '7' | '8' | '9' ;
	digits  = digit09 , { digit09 } ;
	@param c der aktuell in Arbeit befindliche char
	@return die geparste Number
	@throws ParseErrorEx 
	*/// fse, 18.09.2015
	
	private String getNumber(String c) throws ParseErrorEx
	{
		String sym = "";

		if (c.equals("-") || Character.isDigit(c.charAt(0)))
		{
			sym += getInt(c);
			c = getCurrCharAndIncr(false);
		}
		if (c.equals("."))
		{
			sym += getFrac(c);
			c = getCurrCharAndIncr(false);
		}
		if (c.equals("E") || c.equals("e"))
			sym += getExp(c);
		else
			mPos--;
		return sym;
	}

	//------------------------ ------------------------- ------------------------
	/**
	Parsed einen int:
	int = [ '-' ] , digit | digit19 , digits ;
	@param c der aktuell in Arbeit befindliche char
	@return Der gelesene Int-Wert als String.
	@throws ParseErrorEx 
	*/// fse, 18.09.2015
	
	private String getInt(String c) throws ParseErrorEx
	{
		String innt = "";

		if (c.equals("-"))
		{
			innt += c;
			c = getCurrCharAndIncr(false);
		}
		if (c.equals("0"))
		{
			innt += c;
			c = getCurrCharAndIncr(false);
			if (Character.isDigit(c.charAt(0)))
				throw new ParseErrorEx(mJsonStr, mPos, "Digit 1 - 9 expected!");
			else
			{
				mPos--;
				return innt;
			}
		}
		innt += getDigits(c);
		return innt;
	}

	//------------------ ------------------ ------------------ ------------------
	/**
	Parsed eine Ziffernfolge:
	digits = digit09 , { digit09 } ;
	@param c der aktuell in Arbeit befindliche char
	@param ds der aufzubauende String aus Ziffern
	*/// fse, 20.09.2015

	private String getDigits(String c) throws ParseErrorEx
	{
		String digits = "";

		if (!Character.isDigit(c.charAt(0)))
			throw new ParseErrorEx(mJsonStr, mPos, "Digit expected!");
		while (Character.isDigit(c.charAt(0)))
		{
			digits += c;
			c = getCurrCharAndIncr(false);
		}
		mPos--;
		return digits;
	}

	//------------------------ ------------------------- ------------------------
	/**
	Parsed eine Zeichenfolge bestehend aus einem fuehrenden "." und weiteren
	Ziffern.
	frac = '.' , digits ;
	@param c der aktuell in Arbeit befindliche char
	@return frac der aufzubauende String aus Dezimalpunkt und Ziffern
	*/// fse, 18.09.2015

	private String getFrac(String c) throws ParseErrorEx
	{
		String frac = "";

		if (!c.equals("."))
			throw new ParseErrorEx(mJsonStr, mPos, "'.' expected!");
		frac += c;
		c = getCurrCharAndIncr(false);
		if (!Character.isDigit(c.charAt(0)))
			throw new ParseErrorEx(mJsonStr, mPos, "Digit 0-9 expected!");
		frac += getDigits(c);
		return frac;
	}

	//------------------------ ------------------------- ------------------------
	/**
	Parsed eine exp-Komponente:
	exp = eE , digits
	eE  = 'e' | 'E' [ '+' | '-' ] ;
	@param c der aktuell in Arbeit befindliche char
	@return frac der aufzubauende String aus Dezimalpunkt und Ziffern
	*/// fse, 18.09.2015

	private String getExp(String c) throws ParseErrorEx
	{
		String exp = "" + getEe(c);

		c = getCurrCharAndIncr(false);
		exp += getDigits(c);
		return exp;
	}

	//------------------------ ------------------------- ------------------------
	/**
	Parsed eine Ee-Komponente:
	eE = 'e' | 'E' [ '+' | '-' ] ;
	@param c der aktuell in Arbeit befindliche char
	@return Ee der aufzubauende String aus Dezimalpunkt und Ziffern
	*/// fse, 18.09.2015

	private String getEe(String c) throws ParseErrorEx
	{
		String ee = "";

		if (!c.equals("E") && !c.equals("e"))
			throw new ParseErrorEx(mJsonStr, mPos, "'e' or 'E' expected!");
		ee += c;
		c = getCurrCharAndIncr(false);
		if (c.equals("+") || c.equals("-"))
			ee += c;
		else
			mPos--;
		return ee;
	}
}

//---------------------------------------------------------------------------
//------------------------------------- -------------------------------------
//------------------------ ------------------------- ------------------------
//------------------ ------------------ ------------------ ------------------
//-------------- -------------- --------------- -------------- --------------
