//------------------------------------------------------------------------------
// JsonFse - Copyright (c) 2015 by Friedemann Seebass, Germany.
// Diese Bibliothek ist freie Software. Siehe GNU LGPL Notiz in der Datei
// JsonParser.java.
// This library is free software. See the GNU LGPL notice in file
// JsonParser.java.
//------------------------------------------------------------------------------

package JsonFse;

//------------------------------------------------------------------------------
/**
Base class of all nodes a JSON tree consists of.
@author Friedemann Seebass
*/// fse, 26.09.2015

public abstract class Node
{
	int    mTabWidth = 3; 
	String mNl       = System.getProperty("line.separator");

	public String getAsString() {	return getAsString(0, false); }

	//---------------------------------------------------------------------------
	/**
	In JSON all strings are surrounded by quotation marks. These are regularly
	superfluous for further processing. This method returns the string without
	these quotes if the Node is a NodeString.
	@return The string without the leading and trailing quotes.
	*/// fse, 07.11.2015

	public String getAsStrippedString()
	{
		if (this instanceof NodeString)
			return ((NodeString)this).getAsStrippedString();
		else
			return getAsString();
	}
	
	//---------------------------------------------------------------------------
	/**
	Delivers recursively a node and all of its children as a formatted string.
	Child nodes and their elements are indented according to the following rules:
	<ul>
	<li> Each pair (of label and value) is in its own line.
	<li> Values containing sub elements (i. e. JSON objects and arrays) start in
	     the same line as the label.
	<li> If they contain no or only one element they are written in the same
	     line, following directly after the label.
	<li> If they contain more than one element the opening bracket is written into
	     the same line as the label, but each element in its own line. The
	     elements are indented further mTabWidth spaces. The closing bracket is
	     written in its own line, but in the same column as the opening bracket.
	</ul>
	@param columnOffset Number of spaces an element is indented.
	@param hasLabel     If true this node starts in a line with a label. The
	                    element must not be indented then but follows immediately
	                    after the label.
	@return             Formatted string of the node and its childs.
	*/// fse, 26.09.2015

	public String getAsString(int columnOffset, boolean hasLabel)
	{
		return "not implemented!";
	}

	//---------------------------------------------------------------------------
	/**
	Delivers a string consisting out of num spaces.
	@param num Number of spaces.
	@return    String consisting out of num spaces.
	*/
	protected String indent(int num)
	{
		String spaces = "";

		for (int i = 0; i < num; i++)
			spaces += " ";
		return spaces;
	}
	
	//---------------------------------------------------------------------------
	/**
	If the node is of type NodeObj or NodeArray getNext() delivers the next
	element (of type NodePair or NodeValue). The index the element is read from
	can be set with setIdx(). The index is incremented on each call
	automatically.
	@return The next element in a list of NodePairs or NoveValues or null if no
	        more elements are available.
	*/// fse, 11.10.2015

	public Node getNext()
	{
		System.out.println("getNext() not available for this kind of node!");
		return null;
	}

	//---------------------------------------------------------------------------
	/**
	If the node is of type NodeObj or NodeArray get() delivers the element with
	index idx. 
	@param idx Index of the wanted NodePair or NodeValue out of a list of these
	           elements.
	@return The element with index idx.
	*/// fse, 11.10.2015

	public Node get(int idx)
	{
		System.out.println("get() not available for this kind of node!");
		return null;
	}
}

//---------------------------------------------------------------------------
//------------------------------------- -------------------------------------
//------------------------ ------------------------- ------------------------
//------------------ ------------------ ------------------ ------------------
//-------------- -------------- --------------- -------------- --------------
