//------------------------------------------------------------------------------
// JsonFse - Copyright (c) 2015 by Friedemann Seebass, Germany.
// Diese Bibliothek ist freie Software. Siehe GNU LGPL Notiz in der Datei
// JsonParser.java.
// This library is free software. See the GNU LGPL notice in file
// JsonParser.java.
//------------------------------------------------------------------------------

package JsonFse;

//------------------------------------------------------------------------------
/**
Node representing an array of values.
@author Friedemann Seebass
*/// fse, 20.10.2015

public class NodeArray extends NodeValue
{
	/** Holds all elements of the array. */
	protected NodeElements mNodeElements = new NodeElements();;
	
	//---------------------------------------------------------------------------
	/**
	Appends the value to the end of the element list.
	@param nv The value to add.
	*/// fse, 22.12.2016
	
	public void add(NodeValue nv)
	{
		if (mNodeElements == null)
			mNodeElements = new NodeElements();
		mNodeElements.add(nv);
	}
	
	//---------------------------------------------------------------------------

	@Override
	public String getAsString(int columnOffset, boolean hasLabel)
	{
		String str = hasLabel ? "[" : indent(columnOffset) + "[";
		String elements;
		
		if (mNodeElements == null)
		{
			str += "]";
			return str;
		}
		elements = mNodeElements.getAsString(0, hasLabel);
		
		if (elements.contains(mNl))
			str += mNl + mNodeElements.getAsString(columnOffset+mTabWidth, hasLabel) + mNl + indent(columnOffset) + "]";
		else
			str += " " + mNodeElements.getAsString(0, hasLabel) + " ]";
		return str;
	}

	//---------------------------------------------------------------------------
	/**
	Sets the index used by the getNext() method.
	@param idx index where the next getNext() call is reading from
	*/// fse, 18.10.2015
	
	public void setIdx(int idx)
	{
		if (mNodeElements != null)
			mNodeElements.mIdx = idx;
	}

	//---------------------------------------------------------------------------
	@Override
	public NodeValue getNext() { return mNodeElements == null ? null : mNodeElements.getNext(); }

	//---------------------------------------------------------------------------
	@Override
	public NodeValue get(int idx) { return mNodeElements == null ? null : mNodeElements.get(idx); }

	//---------------------------------------------------------------------------
	/**
	Returns the number of elements in the NodeArray.
	@return  The number of elements in this object.
	*/// 22.12.2015

	public int size()
	{
		return mNodeElements.size();
	}

}

//---------------------------------------------------------------------------
//------------------------------------- -------------------------------------
//------------------------ ------------------------- ------------------------
//------------------ ------------------ ------------------ ------------------
//-------------- -------------- --------------- -------------- --------------
