//------------------------------------------------------------------------------
// JsonFse - Copyright (c) 2015 by Friedemann Seebass, Germany.
// Diese Bibliothek ist freie Software. Siehe GNU LGPL Notiz in der Datei
// JsonParser.java.
// This library is free software. See the GNU LGPL notice in file
// JsonParser.java.
//------------------------------------------------------------------------------

package JsonFse;

//------------------------------------------------------------------------------

import java.util.ArrayList;

//------------------------------------------------------------------------------
/**
Holds the elements of a NodeArray.
@author Friedemann Seebass
*/// fse, 20.10.2015

public class NodeElements extends Node
{
	protected ArrayList<NodeValue> mValueList = new ArrayList<>();
	protected int                  mIdx       = 0;

	//---------------------------------------------------------------------------
	/**
	Appends the value to the end of the element list.
	@param nv The value to add.
	*/// fse, 22.12.2016
	
	protected void add(NodeValue nv)
	{
		mValueList.add(nv);
	}
	
	//---------------------------------------------------------------------------

	@Override
	public String getAsString(int columnOffset, boolean hasLabel)
	{
		String str  = "";
		int    size = mValueList.size();
		
		if (size > 0)
			str +=              mValueList.get(0).getAsString(columnOffset, false);
		for (int i = 1; i < size; i++)
			str += ", " + mNl + mValueList.get(i).getAsString(columnOffset, false);
		return str;
	}

	//---------------------------------------------------------------------------
	/**
	Sets the index used by the getNext() method.
	@param idx index where the next getNext() call is reading from
	*/// fse, 18.10.2015
	
	public void setIdx(int idx)
	{
		mIdx = idx;
	}

	//---------------------------------------------------------------------------
	/**
	Returns the next NodeValue element from mValueList.
	@return The next NodeValue element from mValueList.
	*/// fse, 11.10.2015

	@Override
	public NodeValue getNext()
	{
		return mIdx < mValueList.size() ? mValueList.get(mIdx++) : null;
	}
	
	//---------------------------------------------------------------------------

	@Override
	public NodeValue get(int idx)
	{
		NodeValue nv = null;

		if (idx >= 0 && idx < mValueList.size())
			nv = mValueList.get(idx);
		return nv;
	}

	//---------------------------------------------------------------------------
	/**
	Returns the number of elements in an object of NodeElements.
	@return  The number of elements in this object.
	*/// 22.12.2015

	public int size()
	{
		return mValueList.size();
	}
}

//---------------------------------------------------------------------------
//------------------------------------- -------------------------------------
//------------------------ ------------------------- ------------------------
//------------------ ------------------ ------------------ ------------------
//-------------- -------------- --------------- -------------- --------------
