//------------------------------------------------------------------------------
// JsonFse - Copyright (c) 2015 by Friedemann Seebass, Germany.
// Diese Bibliothek ist freie Software. Siehe GNU LGPL Notiz in der Datei
// JsonParser.java.
// This library is free software. See the GNU LGPL notice in file
// JsonParser.java.
//------------------------------------------------------------------------------

package JsonFse;

import java.util.ArrayList;

//------------------------------------------------------------------------------
/**
 This node class represents the members (i. e. the pairs) of a JSON object.
 @author Friedemann Seebass
 */// 25.10.2015

public class NodeMembers extends Node
{
	protected ArrayList<NodePair> mPairList = new ArrayList<>();
	protected int                 mIdx      = 0; // Index in der Elementliste

	//---------------------------------------------------------------------------

	@Override
	public String getAsString(int columnOffset, boolean hasLabel)
	{
		int      size = mPairList.size();
		String   str  = "";
		NodePair np;
		
		if (size > 0)
		{
			np = mPairList.get(0);
			str = np.getAsString(columnOffset, hasLabel);
		}
		for (int i = 1; i < size; i++)
		{
			np = mPairList.get(i);
			str += "," + mNl + np.getAsString(columnOffset, hasLabel);
		}
		return str;
	}

	//---------------------------------------------------------------------------
	/**
	Sets the index used by the getNext() method.
	@param idx index where the next getNext() call is reading from
	*/// fse, 18.10.2015
	
	public void setIdx(int idx)
	{
		mIdx = idx;
	}

	//---------------------------------------------------------------------------
	/**
	Returns the next NodePair element from mPairList, incrementing mIdx
	afterwards.
	@return the NodePair with index mIdx in mPairList
	*/// fse, 11.10.2015

	@Override
	public NodePair getNext()
	{
		NodePair np = null;
		
		if (mIdx < mPairList.size())
			np = mPairList.get(mIdx);
		mIdx++;
		return np;
	}
	
	//---------------------------------------------------------------------------
	/**
	Returns the NodePair element with the given index from mPairList.
	@param idx index of the wanted element
	@return das mPairList[idx]
	*/// fse, 11.10.2015

	@Override
	public NodePair get(int idx)
	{
		NodePair np = null;
		
		if (idx >= 0 && idx < mPairList.size())
			np = mPairList.get(idx);
		return np;
	}

	//---------------------------------------------------------------------------
	/**
	Adds a new pair to the end of the list.
	@param np the pair to add
	*/// fse, 21.12.2016

	public void add(NodePair np)
	{
		mPairList.add(np);
	}
	
}

//---------------------------------------------------------------------------
//------------------------------------- -------------------------------------
//------------------------ ------------------------- ------------------------
//------------------ ------------------ ------------------ ------------------
//-------------- -------------- --------------- -------------- --------------
