//------------------------------------------------------------------------------
// JsonFse - Copyright (c) 2015 by Friedemann Seebass, Germany.
// Diese Bibliothek ist freie Software. Siehe GNU LGPL Notiz in der Datei
// JsonParser.java.
// This library is free software. See the GNU LGPL notice in file
// JsonParser.java.
//------------------------------------------------------------------------------

package JsonFse;

//------------------------------------------------------------------------------
/**
 Represents a whole JSON object, i. e. its members, i. e. a list of pairs
 consisting out of a label and a value.
 @author Friedemann Seebass
*/// fse, 11.10.2015

public class NodeObj extends NodeValue
{
	protected NodeMembers mNodeMembers = new NodeMembers();
	
	//---------------------------------------------------------------------------
	
	@Override
	public String getAsString(int columnOffset, boolean hasLabel)
	{
		String str = hasLabel ? "{" : indent(columnOffset) + "{";
		String members;
		
		if (mNodeMembers == null)
		{
			str += "}";
			return str;
		}
		members = mNodeMembers.getAsString(0, hasLabel);
		if (members.contains(mNl))
			str += mNl + mNodeMembers.getAsString(columnOffset+mTabWidth, true) + mNl + indent(columnOffset) + "}";
		else
			str += " " + mNodeMembers.getAsString(0, hasLabel) + " }";
		return str;
	}

	//---------------------------------------------------------------------------
	/**
	Sets the index used by the getNext() method.
	@param idx index where the next getNext() call is reading from
	*/// fse, 03.07.2016
	
	public void setIdx(int idx)
	{
		mNodeMembers.mIdx = idx;
	}

	//---------------------------------------------------------------------------
	/**
	Returns the next NodePair element from mNodeMembers.
	@return The next NodePair element from mNodeMembers.
	*/// fse, 11.10.2015

	@Override
	public NodePair getNext() { return mNodeMembers.getNext(); }
	
	//---------------------------------------------------------------------------
	/**
	Returns the element with index idx from mNodeMembers.
	@param idx Index of the wanted element in mNodeMembers.
	@return The element with index idx from mNodeMembers.
	*/// fse, 11.10.2015

	@Override
	public NodePair get(int idx) { return mNodeMembers.get(idx); }
	
	//---------------------------------------------------------------------------
	/**
	Gets the value of the pair with the string (i. e. the label) given in the
	parameter. The search starts at index 0, and only the first pair is found if
	there should be more than one with the same label.
	@param string The label to search.
	@return The value of the label or null if not found.
	*/// fse, 28.12.2015

	public NodeValue getValue(String string)
	{
		int       idx   = mNodeMembers.mIdx;
		NodeValue value = null;
		NodePair  np;
		
		if (string.charAt(0) != '\"')
			string = "\"" + string + "\"";
		mNodeMembers.mIdx = 0;
		np = getNext();
		while (np != null)
		{
			if (np.mString.getAsString().equals(string))
			{
				value = np.mValue;
				break;
			}
			np = getNext();
		}
		mNodeMembers.mIdx = idx;
		return value;
	}

	//---------------------------------------------------------------------------
	/**
	Adds a new pair to the end of the member list.
	@param np the pair to add
	*/// fse, 21.12.2016

	public void add(NodePair np)
	{
		mNodeMembers.add(np);
	}

}

//---------------------------------------------------------------------------
//------------------------------------- -------------------------------------
//------------------------ ------------------------- ------------------------
//------------------ ------------------ ------------------ ------------------
//-------------- -------------- --------------- -------------- --------------
