//------------------------------------------------------------------------------
// JsonFse - Copyright (c) 2015 by Friedemann Seebass, Germany.
// Diese Bibliothek ist freie Software. Siehe GNU LGPL Notiz in der Datei
// JsonParser.java.
// This library is free software. See the GNU LGPL notice in file
// JsonParser.java.
//------------------------------------------------------------------------------

package JsonFse;

//------------------------------------------------------------------------------
/**
 Represents a pair consisting of a label and a value.
 @author fs
 */
public class NodePair extends Node
{
	/** The label of the pair. */
	protected NodeString mString;
	
	/** The value of the pair. */
	protected NodeValue  mValue;
	
	public NodeString getString  () { return mString; }
	public NodeValue getValue() { return mValue; }

	public NodePair() {}
	
	//---------------------------------------------------------------------------
	/**
	Constructor for creating a pair containing a string as value.
	The additional quotes needed by the JSON syntax for enclosing the strings are
	added automatically and must not	be given.
	@param string The label of the pair.
	@param value  The value of the pair.
	*/// fse, 21.12.2016
	
	public NodePair(String string, String value)
	{
		mString = new NodeString(string);
		mValue  = new NodeString(value);
	}
	
	//---------------------------------------------------------------------------
	/**
	Constructor for creating a pair containing a number as value.
	The additional quotes needed by the JSON syntax for enclosing the label are
	added automatically and must not	be given.
	@param string The label of the pair.
	@param value  The value of the pair.
	*/// fse, 22.12.2016
	
	public NodePair(String string, double value)
	{
		mString = new NodeString(string);
		mValue  = new NodeNumber(Double.toString(value));
	}
	
	//---------------------------------------------------------------------------
	/**
	Constructor for creating a pair containing a boolean as value.
	The additional quotes needed by the JSON syntax for enclosing the label are
	added automatically and must not	be given.
	@param string The label of the pair.
	@param value  The value of the pair.
	*/// fse, 22.12.2016
	
	public NodePair(String string, boolean value)
	{
		mString = new NodeString(string);
		if (value)
			mValue  = new NodeTrue();
		else
			mValue  = new NodeFalse();
	}
	
	//---------------------------------------------------------------------------
	/**
	Constructor for creating a pair containing a null as value.
	The additional quotes needed by the JSON syntax for enclosing the label are
	added automatically and must not	be given.
	@param string The label of the pair.
	*/// fse, 22.12.2016
	
	public NodePair(String string)
	{
		mString = new NodeString(string);
		mValue  = new NodeNull();
	}
	
	//---------------------------------------------------------------------------
	/**
	Constructor for creating a pair containing a NodeArray as value.
	The additional quotes needed by the JSON syntax for enclosing the label are
	added automatically and must not	be given.
	@param string The label of the pair.
	@param value  The value of the pair.
	*/// fse, 22.12.2016
	
	public NodePair(String string, NodeArray value)
	{
		mString = new NodeString(string );
		mValue  = value;
	}
	
	//---------------------------------------------------------------------------
	/**
	Constructor for creating a pair containing a NodeObj as value.
	The additional quotes needed by the JSON syntax for enclosing the label are
	added automatically and must not	be given.
	@param string The label of the pair.
	@param value  The value of the pair.
	*/// fse, 22.12.2016
	
	public NodePair(String string, JsonObj value)
	{
		mString = new NodeString(string);
		mValue  = value.mNodeObj;
	}
	
	//---------------------------------------------------------------------------

	@Override
	public String getAsString(int columnOffset, boolean hasLabel)
	{
		String string            = mString.getAsString(columnOffset, hasLabel);
		String colon             = " : ";
		int    stringLen         = string.length();
		int    valueColumnOffset = columnOffset + stringLen + colon.length();
		String str               = indent(columnOffset) + string + " : " + mValue.getAsString(valueColumnOffset, true);

		return str;
	}
	
}

//---------------------------------------------------------------------------
//------------------------------------- -------------------------------------
//------------------------ ------------------------- ------------------------
//------------------ ------------------ ------------------ ------------------
//-------------- -------------- --------------- -------------- --------------
