//------------------------------------------------------------------------------
// JsonFse - Copyright (c) 2015 by Friedemann Seebass, Germany.
// Diese Bibliothek ist freie Software. Siehe GNU LGPL Notiz in der Datei
// JsonParser.java.
// This library is free software. See the GNU LGPL notice in file
// JsonParser.java.
//------------------------------------------------------------------------------

package JsonFse;

//------------------------------------------------------------------------------
/**
This exception is thrown when the JSON parser detects a parse error. The error
position and the cause of the error can be seen by calling the methods
getErrStr(), getErrPos() and getErrMsg().
@author Friedemann Seebass
*/// 25.10.2015

public class ParseErrorEx extends Exception
{
	/** The JSON string producing the error. */
	private String mJsonStr;
	
	/** Error position in the JSON object (it must be given as string). */
	private int    mPos;
	
	/** The error message describing the error. */
	private String mErrStr; ///< der Fehler-String
	
	/** Platform independant Newline. */
	private String mNl = System.getProperty("line.separator");


	//---------------------------------------------------------------------------
	/**
	Returns the error position in the JSON object.
	@return The error position in the JSON object.
	*/// fse, 25.10.2015
	
	public int getErrPos() { return mPos-1; }
	
	//---------------------------------------------------------------------------
	/**
	Returns the error message.
	@return The error message.
	*/// 25.10.2015

	public String getErrStr() { return mErrStr; }

	//---------------------------------------------------------------------------
	/**
	Creates an empty ParseError exception.
	*/// fse, 28.12.2016
	
	public ParseErrorEx()
	{
		super();
	}

	//---------------------------------------------------------------------------
	/**
	Creates a ParseError exception and sets the members for showing the error
	position and the cause of the error.
	@param jsonStr The original string to parse causing the error.
	@param pos     The index in jsonStr where the error is detected.
	@param errStr  A string describing the cause of the error.
	*/// fse, 28.12.2016

	protected ParseErrorEx(String jsonStr, int pos, String errStr)
	{
		super(errStr);
		mJsonStr = jsonStr;
		mPos     = pos;
		mErrStr  = errStr;
	}
	
	//---------------------------------------------------------------------------
	/**
	Returns a multi line error message string with the original string to parse
	in the first line and a pointer where the error was detected in the second
	line and the error in the last line.
	@return The error message.
	*/// fse, 06.12.2015

	public String getErrMsg()
	{
		String        msg = "";
		StringBuilder buf = new StringBuilder();
		int           errPos = mPos-1;
		
		for (int i = 0; i < errPos; i++)
			buf.append('-');
		buf.append('^').append(" errPos: ").append(errPos);
		msg += mJsonStr + mNl;
		msg += buf + mNl;
//		msg += mJsonStr.substring(0, mPos) + mNl;
		msg += mErrStr;
		return msg;
	}
}

//---------------------------------------------------------------------------
//------------------------------------- -------------------------------------
//------------------------ ------------------------- ------------------------
//------------------ ------------------ ------------------ ------------------
//-------------- -------------- --------------- -------------- --------------
