    /* * To change this license header, choose License Headers in
    Project Properties. * To change this template file, choose Tools |
    Templates * and open the template in the editor. */
    /**
    This package provides the classes of a very simple JSON parser. <br>
    <h2>Introduction</h2>
    JSON (<u>J</u>ava<u>S</u>cript <u>O</u>bject <u>N</u>otation) is a
    lightweight data-interchange format. Basically a JSON object is a
    comma separated list of pairs, enclosed in curly braces. A pair
    consists out of a label and a value, separated by a colon. E. g.:<br>
    <br>
    {<br>
    &nbsp;&nbsp;&nbsp; "name" : "Mike",<br>
    &nbsp;&nbsp;&nbsp; "age" : 23,<br>
    &nbsp;&nbsp;&nbsp; "hobbies" : [ "fishing", "rugby", "computer
    games" ]<br>
    }<br>
    <br>
    The value can hold different kind of data, e. g. strings, numbers or
    even arrays. This seems very easy but can become rather complicated
    if a value is another JSON object: We have a recursive structure
    then, and a JSON object is best represented by a tree of nodes
    therefore.<br>
    <b>Attention</b>: All strings (and the labels) must be enclosed in
    quotes, all other types must not.<br>
    See <a href="http://www.json.org/">www.json.org</a> for details.
    <h2>Parsing a JSON String and Traversing through the Parse Tree</h2>
    To parse a JSON string and build the tree it must be given as a Java
    <code> String</code> to the constructor of the {@link
    JsonFse.JsonObj} class. Example:<br>
    <br>
    <code>&nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; String s = "{" +<br>
      &nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp;
      &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;

      "\"name\" : \"Mike\"," +<br>
      &nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp;
      &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;

      "\"age\"&nbsp; : 23,"&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; +<br>
      &nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp;
      &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;

      "\"hobbies\" : [ \"fishing\", \"rugby\", \"computer games\" ]" +<br>
      &nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp;
      &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; "}";<br>
      &nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; try<br>
      &nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; {<br>
      &nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; JsonObj
      jo = new JsonObj(s);<br>
      &nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp;
      System.out.println(jo.getAsString());<br>
      &nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; }<br>
      &nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; catch (ParseErrorEx ex)<br>
      &nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; {<br>
      &nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp;
      System.out.println(ex.getErrMsg());<br>
      &nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; }<br>
    </code><br>
    {@link JsonFse.JsonObj#getAsString} retranslates the tree
    recursively into a formatted string. If the original string could
    not be parsed because of an error a {@link JsonFse.ParseErrorEx}
    exception is thrown. {@link JsonFse.ParseErrorEx#getErrPos} delivers
    the index in the string where the error is detected.<br>
    <br>
    To traverse through the <code>JsonObj</code> tree some knowledge of
    the node types is necessary. Their names reflect the names given in
    <a href="http://www.json.org/">www.json.org</a>. The abstract base
    class is {@link JsonFse.Node}, {@link JsonFse.NodePair} represents a
    pair, {@link JsonFse.NodeNumber} a number etc. To get pairs in a
    JSON object there are the methods {@link JsonFse.JsonObj#getNext}
    and {@link JsonFse.JsonObj#get}. The same methods are available in
    the {@link JsonFse.NodeArray} class. Example traversing through the
    JSON object <code>jo</code> created above:<br>
    <br>
    <code>&nbsp;&nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; NodeString ns,
      nsa;</code><code><br>
    </code><code>&nbsp;&nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; NodeValue
      nv;</code><code><br>
    </code><code>&nbsp;&nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; NodePair np
      = jo.getNext();</code><code><br>
    </code><code>&nbsp;&nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; while (np
      != null)</code><code><br>
    </code><code>&nbsp;&nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; {</code><code><br>
    </code><code>&nbsp;&nbsp; &nbsp; &nbsp;&nbsp;&nbsp;
      &nbsp;&nbsp;&nbsp; ns = np.getString();</code><code><br>
    </code><code>&nbsp; &nbsp;&nbsp; &nbsp;&nbsp;&nbsp;
      &nbsp;&nbsp;&nbsp; System.out.println("label: " +
      ns.getAsString());</code><code><br>
    </code><code>&nbsp;&nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp;
      &nbsp;&nbsp;&nbsp; nv = np.getValue();</code><code><br>
    </code><code>&nbsp;&nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp;
      &nbsp;&nbsp;&nbsp; if (nv instanceof NodeArray)</code><code><br>
    </code><code>&nbsp;&nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp;
      &nbsp;&nbsp;&nbsp; {</code><code><br>
    </code><code>&nbsp;&nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp;
      &nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; nsa = (NodeString)nv.get(1);</code><code><br>
    </code><code>&nbsp;&nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp;
      &nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; System.out.println("Second
      hobby of Mike: " + nsa.getAsString());</code><code><br>
    </code><code>&nbsp;&nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp;
      &nbsp;&nbsp;&nbsp; }</code><code><br>
    </code><code>&nbsp;&nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp;
      &nbsp;&nbsp;&nbsp; np = jo.getNext();</code><code><br>
    </code><code>&nbsp;&nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; }</code><code><br>
    </code><code>&nbsp;&nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp;
      System.out.println("First pair in jo: " +
      jo.get(0).getAsString());</code><code><br>
    </code><br>
    This produces the output:<br>
    <br>
    <code>label: "name"</code><code><br>
    </code><code>label: "age"</code><code><br>
    </code><code>label: "hobbies"</code><code><br>
    </code><code>Second hobby of Mike: "rugby"</code><code><br>
    </code><code>First pair in jo: "name" : "Mike"</code><code><br>
    </code><br>
    As you can see it is easy to get a special value in a JSON object or
    of an array in it if you know the structure of the JSON object. If
    you do not know the structure you need to recognize it while
    traversing through the structure. Use the Java command <code>instanceof</code>
    to detect the type of unknown values as shown above.<br>
    Hint: There is no class <code>NodeLabel</code>, its name is {@link
    JsonFse.NodeString} instead, because of the wording in <a
      href="http://www.json.org/">www.json.org</a>.<br>
    <h2> Creating JSON Objects</h2>
    <p>It is possible to build up a JSON object by adding single pairs
      to it. The following code creates the same object as shown above:<br>
    </p>
    <p><code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; // Create
        an empty JSON object:<br>
        &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; JsonObj jo =
        new JsonObj();</code></p>
    <p><code>&nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp;&nbsp; // Create the
        NodeArray:</code><code><br>
      </code><code>&nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp;&nbsp; NodeArray
        na = new NodeArray();</code><code><br>
      </code><code>&nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp;&nbsp;
        na.add(new NodeString("fishing"));</code><code><br>
      </code><code>&nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp;&nbsp;
        na.add(new NodeString("rugby"));</code><code><br>
      </code><code>&nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp;&nbsp;
        na.add(new NodeString("computer games"));</code></p>
    <p><code>&nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp;&nbsp; // Add the
        pairs:</code><code><br>
      </code><code>&nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp;&nbsp;
        jo.addPair("name", "Mike");</code><code><br>
      </code><code>&nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp;&nbsp;
        jo.addPair("age", 23);</code><code><br>
      </code><code>&nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp;&nbsp;
        jo.addPair("hobbies", na);</code></p>
    <p><code>&nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp;&nbsp;
        System.out.println(jo.getAsString());</code><code><br>
      </code></p>
    <p>When creating <code>NodeString</code>s or adding pairs
      containing strings it is not necessary to add the extra quotes
      around the strings; this is done by the constructor <code>NodeString()</code>
      automatically.<br>
    </p>
    <h2>Grammar</h2>
    The implementation is able to parse JSON objects according to this
    grammar, notated in EBNF:<br>
    <br>
    <code>object&nbsp;&nbsp;&nbsp; = '{' , [ members ] , '}'</code><code>
      ;<br>
    </code><code>members&nbsp;&nbsp; = pair , { ',' , pair }</code><code>
      ;<br>
    </code><code>pair&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; = string , ':' ,
      value</code><code> ;</code><br>
    <code>string&nbsp;&nbsp;&nbsp; = '"' , { char } , '"' ;</code><br>
    <code>char&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; = ? any Unicode character ?
      - ctrlChar - '\') | ctrlChar | hexNumber</code><code> ;<br>
    </code><code>ctrlChar&nbsp; = '\"' | '\\' | '\/' | '\b' | '\f' |
      '\n' | '\r' | '\t'</code><code> ;<br>
    </code><code>hexNumber = '&#92;u' , hexDigit , hexDigit , hexDigit ,
      hexDigit</code><code> ;<br>
    </code><code>hexDigit&nbsp; = digit09 | hexChar</code><code> ;<br>
    </code><code>digit09&nbsp;&nbsp; = digit0 | digit19</code><code> ;<br>
    </code><code>digit0&nbsp;&nbsp;&nbsp; = '0'</code><code> ;<br>
    </code><code>digit19&nbsp;&nbsp; = '1' | '2' | '3' | '4' | '5' | '6'
      | '7' | '8' | '9'</code><code> ;<br>
    </code><code>hexChar&nbsp;&nbsp; = 'A' | 'B' | 'C' | 'D' | 'E' | 'F'
      | 'a' | 'b' | 'c' | 'd' | 'e' | 'f'</code><code> ;<br>
    </code><code>value&nbsp;&nbsp;&nbsp;&nbsp; = string | number |
      object | array | 'true' | 'false' | 'null'</code><code> ;<br>
    </code><code>number&nbsp;&nbsp;&nbsp; = int | int frac | int exp |
      int , frac , exp</code><code> ;<br>
    </code><code>int&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; = [ '-' ] ,
      digit | digit19 , digits</code><code> ;<br>
    </code><code>frac&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; = '.' , digits</code><code>
      ;<br>
    </code><code>exp&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; = eE , digits</code><code><br>
    </code><code>digits&nbsp;&nbsp;&nbsp; = digit09 , { digit09 }</code><code>
      ;<br>
    </code><code>eE &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; = 'e' | 'E' [
      '+' | '-' ]</code><code> ;<br>
    </code><code>array&nbsp;&nbsp;&nbsp;&nbsp; = '[' , [ elements ] ,
      ']'</code><code> ;<br>
    </code><code>elements&nbsp; = value , { ',' , value }</code><code> ;<br>
    </code><br>
    Reminder: The used symbols of the EBNF meta language are:<br>
    <br>
    <table summary="EBNF meta language symbols" border="1"
      cellpadding="2" cellspacing="2">
      <tbody>
        <tr>
          <th valign="top">Symbol<br>
          </th>
          <th valign="top">Meaning<br>
          </th>
        </tr>
        <tr>
          <td align="center" valign="top">=<br>
          </td>
          <td valign="top">definition<br>
          </td>
        </tr>
        <tr>
          <td align="center" valign="top">,<br>
          </td>
          <td valign="top">concatenation<br>
          </td>
        </tr>
        <tr>
          <td align="center" valign="top">;<br>
          </td>
          <td valign="top">termination<br>
          </td>
        </tr>
        <tr>
          <td align="center" valign="top">|<br>
          </td>
          <td valign="top">alternation<br>
          </td>
        </tr>
        <tr>
          <td align="center" valign="top">[ ... ]<br>
          </td>
          <td valign="top">option<br>
          </td>
        </tr>
        <tr>
          <td align="center" valign="top">{ ... }<br>
          </td>
          <td valign="top">repetition<br>
          </td>
        </tr>
        <tr>
          <td align="center" valign="top">' ... ' <br>
          </td>
          <td valign="top">terminal string<br>
          </td>
        </tr>
        <tr>
          <td align="center" valign="top">? ... ?<br>
          </td>
          <td valign="top">special sequence<br>
          </td>
        </tr>
        <tr>
          <td align="center" valign="top">-<br>
          </td>
          <td valign="top">exception<br>
          </td>
        </tr>
      </tbody>
    </table>
    <br>
    <h2>Class Diagram</h2>
    The UML class diagram looks like this (sorry, Inkscape does not
    provide the correct end marker for the "uses" relationship; it
    should be "<code>-&gt;</code>" instead of "<code>-&gt;&gt;&gt;</code>"):<br>
    <br>
    <img alt="UML Class Diagram" src="doc-files/UML.png"> <br>
    <br>
	@author Friedemann Seebass, Germany (<a
      href="http://www.friedemann-seebass.de/">www.friedemann-seebass.de</a>)
	@version 1.0.0
    */
    package JsonFse;
